/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.powershell.resource;

import java.util.List;

import com.redhat.rhevm.api.common.util.LinkHelper;
import com.redhat.rhevm.api.model.Event;
import com.redhat.rhevm.api.model.Events;
import com.redhat.rhevm.api.powershell.model.PowerShellEvent;
import com.redhat.rhevm.api.powershell.util.PowerShellCmd;
import com.redhat.rhevm.api.powershell.util.PowerShellUtils;
import com.redhat.rhevm.api.resource.EventResource;
import com.redhat.rhevm.api.resource.EventsResource;

public class PowerShellEventsResource extends InjectableUriProviderBase implements EventsResource {

    protected List<Event> runAndParse(String command) {
        return PowerShellEvent.parse(getParser(), PowerShellCmd.runCommand(getPool(), command));
    }

    protected Event runAndParseSingle(String command) {
        List<Event> events = runAndParse(command);
        return !events.isEmpty() ? events.get(0) : null;
    }

    @Override
    public Events list() {
        Events ret = new Events();
        for (Event event : runAndParse(getSelectCommand("select-event", getUriInfo(), Event.class))) {
            ret.getEvent().add(LinkHelper.addLinks(getUriInfo(), event));
        }
        return ret;
    }

    @Override
    public EventResource getEventSubResource(String id) {
        return new PowerShellEventResource(id, this);
    }

    Event lookupEvent(String id) {
        StringBuilder buf = new StringBuilder();
        buf.append("select-event");
        buf.append(" | ? ");
        buf.append("{ $_.id -eq ");
        buf.append(PowerShellUtils.escape(id));
        buf.append(" }");
        return LinkHelper.addLinks(getUriInfo(), runAndParseSingle(buf.toString()));
    }
}
