/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.powershell.resource;

import java.util.List;

import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriBuilder;

import com.redhat.rhevm.api.model.IP;
import com.redhat.rhevm.api.model.Network;
import com.redhat.rhevm.api.model.Networks;
import com.redhat.rhevm.api.resource.NetworkResource;
import com.redhat.rhevm.api.resource.NetworksResource;
import com.redhat.rhevm.api.common.util.LinkHelper;
import com.redhat.rhevm.api.powershell.util.PowerShellCmd;
import com.redhat.rhevm.api.powershell.util.PowerShellUtils;

import static com.redhat.rhevm.api.common.util.CompletenessAssertor.validateParameters;


public class PowerShellNetworksResource
    extends AbstractPowerShellCollectionResource<Network, PowerShellNetworkResource>
    implements NetworksResource {

    public List<Network> runAndParse(String command) {
        return PowerShellNetworkResource.runAndParse(getPool(), getParser(), command);
    }

    public Network runAndParseSingle(String command) {
        return PowerShellNetworkResource.runAndParseSingle(getPool(), getParser(), command);
    }

    @Override
    public Networks list() {
        Networks ret = new Networks();
        for (Network network : runAndParse("get-networks")) {
            ret.getNetworks().add(LinkHelper.addLinks(getUriInfo(), network));
        }
        return ret;
    }

    @Override
    public Response add(Network network) {
        validateParameters(network, "name", "dataCenter.id");
        StringBuilder buf = new StringBuilder();

        buf.append("add-network");
        buf.append(" -name " + PowerShellUtils.escape(network.getName()) + "");
        if (network.getDescription() != null) {
            buf.append(" -description " + PowerShellUtils.escape(network.getDescription()));
        }
        buf.append(" -datacenterid " + PowerShellUtils.escape(network.getDataCenter().getId()));

        if (network.getIp() != null) {
            IP ip = network.getIp();

            if (ip.getAddress() != null) {
                buf.append(" -address " + PowerShellUtils.escape(ip.getAddress()));
            }
            if (ip.getNetmask() != null) {
                buf.append(" -netmask " + PowerShellUtils.escape(ip.getNetmask()));
            }
            if (ip.getGateway() != null) {
                buf.append(" -gateway " + PowerShellUtils.escape(ip.getGateway()));
            }
        }

        if (network.getVlan() != null) {
            buf.append(" -vlanid " + Integer.toString(network.getVlan().getId()));
        }

        if (network.isStp() != null && network.isStp()) {
            buf.append(" -stp");
        }

        if (network.isDisplay() != null && network.isDisplay()) {
            buf.append(" -isdisplay $true");
        }

        network = LinkHelper.addLinks(getUriInfo(), runAndParseSingle(buf.toString()));

        UriBuilder uriBuilder = getUriInfo().getAbsolutePathBuilder().path(network.getId());

        return Response.created(uriBuilder.build()).entity(network).build();
    }

    @Override
    public void remove(String id) {
        StringBuilder buf = new StringBuilder();

        buf.append("$n = get-networks;");
        buf.append("foreach ($i in $n) {");
        buf.append("  if ($i.networkid -eq " + PowerShellUtils.escape(id) + ") {");
        buf.append("    remove-network");
        buf.append(" -networkobject $i");
        buf.append(" -datacenterid $i.datacenterid");
        buf.append("  }");
        buf.append("}");

        PowerShellCmd.runCommand(getPool(), buf.toString());

        removeSubResource(id);
    }

    @Override
    public NetworkResource getNetworkSubResource(String id) {
        return getSubResource(id);
    }

    protected PowerShellNetworkResource createSubResource(String id) {
        return new PowerShellNetworkResource(id, getExecutor(), this, shellPools, getParser());
    }
}
