/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.powershell.resource;

import java.util.ArrayList;
import java.util.List;

import com.redhat.rhevm.api.model.Disk;
import com.redhat.rhevm.api.model.Disks;

import com.redhat.rhevm.api.common.resource.UriInfoProvider;
import com.redhat.rhevm.api.common.util.JAXBHelper;
import com.redhat.rhevm.api.common.util.LinkHelper;
import com.redhat.rhevm.api.powershell.model.PowerShellDisk;
import com.redhat.rhevm.api.powershell.util.PowerShellCmd;
import com.redhat.rhevm.api.powershell.util.PowerShellParser;
import com.redhat.rhevm.api.powershell.util.PowerShellPoolMap;
import com.redhat.rhevm.api.powershell.util.PowerShellUtils;
import com.redhat.rhevm.api.resource.ReadOnlyDevicesResource;


public class PowerShellReadOnlyDisksResource extends AbstractPowerShellDevicesResource<Disk, Disks> implements ReadOnlyDevicesResource<Disk, Disks> {

    private String getCommand;

    public PowerShellReadOnlyDisksResource(String parentId,
                                           PowerShellPoolMap shellPools,
                                           PowerShellParser parser,
                                           String getCommand,
                                           UriInfoProvider uriProvider) {
        super(parentId, shellPools, parser, uriProvider);
        this.getCommand = getCommand;
    }

    public List<Disk> runAndParse(String command) {
        List<Disk> ret = new ArrayList<Disk>();
        for (Disk disk : PowerShellDisk.parse(getParser(), parentId, PowerShellCmd.runCommand(getPool(), command))) {
            ret.add(disk);
        }
        return ret;
    }

    public Disk runAndParseSingle(String command) {
        List<Disk> disks = runAndParse(command);

        return !disks.isEmpty() ? disks.get(0) : null;
    }

    @Override
    public List<Disk> getDevices() {
        StringBuilder buf = new StringBuilder();

        buf.append(PowerShellVmsResource.getStorageDomainLookupHack(parentId));

        buf.append("$v = " + getCommand + " " + PowerShellUtils.escape(parentId) + ";");
        buf.append("$v.GetDiskImages()");

        return runAndParse(buf.toString());
    }

    @Override
    public Disk addLinks(Disk disk) {
        disk = JAXBHelper.clone("disk", Disk.class, disk);
        return LinkHelper.addLinks(getUriInfo(), disk);
    }

    @Override
    public Disks list() {
        Disks disks = new Disks();
        for (Disk disk : getDevices()) {
            disks.getDisks().add(addLinks(disk));
        }
        return disks;
    }

    @Override
    public PowerShellReadOnlyDeviceResource<Disk, Disks> getDeviceSubResource(String id) {
        return new PowerShellReadOnlyDeviceResource<Disk, Disks>(this, id);
    }
}
