/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.powershell.resource;

import java.util.List;

import com.redhat.rhevm.api.common.resource.UriInfoProvider;
import com.redhat.rhevm.api.common.util.LinkHelper;
import com.redhat.rhevm.api.model.BaseResource;
import com.redhat.rhevm.api.model.Statistic;
import com.redhat.rhevm.api.model.Statistics;
import com.redhat.rhevm.api.resource.StatisticResource;
import com.redhat.rhevm.api.resource.StatisticsResource;

public class PowerShellStatisticsResource<R extends BaseResource> extends UriProviderWrapper implements StatisticsResource {

    private AbstractStatisticalQuery<R> query;

    protected PowerShellStatisticsResource(UriInfoProvider uriProvider,
                                           AbstractStatisticalQuery<R> query) {
        super(null, null, null, uriProvider);
        this.query = query;
    }

    @Override
    public StatisticResource getStatisticSubResource(String id) {
        return new PowerShellStatisticResource<R>(id, this);
    }

    @Override
    public Statistics list() {
        Statistics statistics = new Statistics();
        List<Statistic> currentStats = query.getStatistics(query.resolve());
        for (Statistic statistic : currentStats) {
            LinkHelper.addLinks(getUriInfo(), query.adopt(statistic), query.getParentType());
        }
        statistics.getStatistics().addAll(currentStats);
        return statistics;
    }
}
