/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.powershell.resource;

import java.util.List;

import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriBuilder;

import com.redhat.rhevm.api.model.Role;
import com.redhat.rhevm.api.model.User;
import com.redhat.rhevm.api.model.Users;
import com.redhat.rhevm.api.powershell.util.PowerShellCmd;
import com.redhat.rhevm.api.powershell.util.PowerShellUtils;
import com.redhat.rhevm.api.resource.UserResource;
import com.redhat.rhevm.api.resource.UsersResource;

import static com.redhat.rhevm.api.common.util.CompletenessAssertor.validateParameters;

public class PowerShellUsersResource extends InjectableUriProviderBase implements UsersResource {

    public List<User> runAndParse(String command) {
        return PowerShellUserResource.runAndParse(getPool(), getParser(), command);
    }

    public User runAndParseSingle(String command) {
        return PowerShellUserResource.runAndParseSingle(getPool(), getParser(), command);
    }

    @Override
    public Users list() {
        Users ret = new Users();
        for (User user : runAndParse(getSelectCommand("select-user", getUriInfo(), User.class, false))) {
            ret.getUsers().add(PowerShellUserResource.addLinks(getUriInfo(), user));
        }
        return ret;
    }

    @Override
    public Response add(User user) {
        validateParameters(user, "userName|id", "roles.id|name");
        StringBuilder buf = new StringBuilder();

        String userArg = null;
        if (user.isSetId()) {
            userArg = PowerShellUtils.escape(user.getId());
        } else {
            buf.append("$u = select-user -AD").append(SEARCH_TEXT);
            buf.append(PowerShellUtils.escape(user.getUserName())).append("; ");
            userArg = "$u.UserId";
        }

        String roleArg = getRoleArg(user.getRoles().getRoles().get(0), buf);

        buf.append("add-user -userid ").append(userArg).append(" -userroleid ").append(roleArg);
        User newUser = PowerShellUserResource.addLinks(getUriInfo(), runAndParseSingle(buf.toString()));
        UriBuilder uriBuilder = getUriInfo().getAbsolutePathBuilder().path(newUser.getId());

        if (user.getRoles().getRoles().size() > 1) {
            for (int i = 1 ; i < user.getRoles().getRoles().size() ; i++) {
                StringBuilder addRole = new StringBuilder();
                String extraRoleArg = getRoleArg(user.getRoles().getRoles().get(i), addRole);
                addRole.append("attach-role -roleid ").append(extraRoleArg)
                       .append(" -elementid ").append(newUser.getId());
                PowerShellRoleResource.runAndParseSingle(getPool(), getParser(), newUser.getId(), addRole.toString());
            }
        }
        return Response.created(uriBuilder.build()).entity(newUser).build();
    }

    @Override
    public void remove(String id) {
        PowerShellCmd.runCommand(getPool(), "remove-user -userid " + PowerShellUtils.escape(id));
    }

    @Override
    public UserResource getUserSubResource(String id) {
        return new PowerShellUserResource(id, executor, shellPools, parser, this);
    }

    static String getRoleArg(Role role, StringBuilder buf) {
        String roleArg = null;
        if (role.isSetId()) {
            roleArg = PowerShellUtils.escape(role.getId());
        } else {
            buf.append("$role = get-roles | ? { $_.Name -eq " + PowerShellUtils.escape(role.getName()) + " }; ");
            roleArg = "$role.Id";
        }
        return roleArg;
    }
}
