/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.powershell.resource;

import java.util.List;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriBuilder;

import com.redhat.rhevm.api.model.VmPool;
import com.redhat.rhevm.api.model.VmPools;
import com.redhat.rhevm.api.resource.VmPoolResource;
import com.redhat.rhevm.api.resource.VmPoolsResource;
import com.redhat.rhevm.api.powershell.util.PowerShellCmd;
import com.redhat.rhevm.api.powershell.util.PowerShellUtils;

import static com.redhat.rhevm.api.common.util.CompletenessAssertor.validateParameters;


public class PowerShellVmPoolsResource
    extends AbstractPowerShellCollectionResource<VmPool, PowerShellVmPoolResource>
    implements VmPoolsResource {

    public List<VmPool> runAndParse(String command) {
        return PowerShellVmPoolResource.runAndParse(getPool(), getParser(), command);
    }

    public VmPool runAndParseSingle(String command) {
        return PowerShellVmPoolResource.runAndParseSingle(getPool(), getParser(), command);
    }

    public VmPool addLinks(VmPool pool) {
        return PowerShellVmPoolResource.addLinks(getUriInfo(), getPool(), getParser(), pool);
    }

    @Override
    public VmPools list() {
        VmPools ret = new VmPools();
        for (VmPool pool : runAndParse(getSelectCommand("select-vmpool", getUriInfo(), VmPool.class))) {
            ret.getVmPools().add(addLinks(pool));
        }
        return ret;
    }

    @Override
    public Response add(VmPool pool) {
        validateParameters(pool, "name", "template.id|name", "cluster.id|name");
        StringBuilder buf = new StringBuilder();

        String templateArg = null;
        if (pool.getTemplate().isSetId()) {
            templateArg = PowerShellUtils.escape(pool.getTemplate().getId());
        } else {
            buf.append("$t = select-template -searchtext ");
            buf.append(PowerShellUtils.escape("name=" + pool.getTemplate().getName()));
            buf.append(";");
            templateArg = "$t.TemplateId";
        }

        String clusterArg = null;
        if (pool.getCluster().isSetId()) {
            clusterArg = PowerShellUtils.escape(pool.getCluster().getId());
        } else {
            buf.append("$c = select-cluster -searchtext ");
            buf.append(PowerShellUtils.escape("name=" +  pool.getCluster().getName()));
            buf.append(";");
            clusterArg = "$c.ClusterId";
        }

        buf.append("add-vmpool");

        buf.append(" -vmpoolname " + PowerShellUtils.escape(pool.getName()));
        if (pool.getDescription() != null) {
            buf.append(" -vmpooldescription " + PowerShellUtils.escape(pool.getDescription()));
        }
        buf.append(" -templateid " + templateArg);
        buf.append(" -hostclusterid " + clusterArg);
        buf.append(" -pooltype Automatic");
        if (pool.getSize() != null) {
            buf.append(" -numofvms " + pool.getSize());
        }

        pool = addLinks(runAndParseSingle(buf.toString()));

        UriBuilder uriBuilder = getUriInfo().getAbsolutePathBuilder().path(pool.getId());

        return Response.created(uriBuilder.build()).entity(pool).build();
    }

    @Override
    public void remove(String id) {
        StringBuilder buf = new StringBuilder();

        buf.append("$p = get-vmpool -vmpoolid " + PowerShellUtils.escape(id) + ";");
        buf.append("remove-vmpool -name $p.name");

        PowerShellCmd.runCommand(getPool(), buf.toString());

        removeSubResource(id);
    }

    @Override
    public VmPoolResource getVmPoolSubResource(String id) {
        return getSubResource(id);
    }

    @Override
    protected PowerShellVmPoolResource createSubResource(String id) {
        return new PowerShellVmPoolResource(id, getExecutor(), this, shellPools, getParser());
    }
}
